--[[
    Core.lua
    
    Main file for Privacy2.
    
    Version: 2.1
    Revision: $Rev: 33172 $
    Author: Sterntraeumer (rewritten by Borlox)
    
    This Addon is originaly coded by Sterntraeumer!
    Diese Addon stammt ursprnglich von Sterntraeumer!
--]]

Privacy = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")

local L = AceLibrary("AceLocale-2.2"):new("Privacy2")
local Waterfall = AceLibrary("Waterfall-1.0")

--[[***************************************]]--

-- heap system for group/raid/vip tables
-- copied from AceDB-2.0

local new, del
do
  local list = setmetatable({}, {__mode="k"})
  function new()
    local t = next(list)
    if t then
      list[t] = nil
      return t
    else
      return {}
    end
  end
  
  function del(t)
    setmetatable(t, nil)
    for k in pairs(t) do
      t[k] = nil
    end
    list[t] = true
  end
end

--[[***************************************]]--

-- some const which must not be changed
local AUTO_ANSWER_LEAD_IN = "[Privacy2]"
local REPLACE_BY_TELL_PREF = "$k"

-- member variables
Privacy.group = nil -- will be a table, but is nil until we join a group
Privacy.raid  = nil
Privacy.guild = nil
Privacy.friends = nil
Privacy.tmpVIP = new() -- list of all persons, who are temporaryly allowed to send whispers
Privacy.BlockedWhispers = new() -- list of all blocked whispers

Privacy.LTSender = "" -- sender of the last whisper
Privacy.LTMsg = "" -- last whisper
Privacy.LTTime = 0 -- last whisper's time

Privacy.LIRec = "" -- same for last whisper-inform
Privacy.LIMsg = ""
Privacy.LITime = 0

Privacy.Revision = strsub("$Rev: 33172 $", 6, -3)

------

Privacy:RegisterDB("PrivacyDB", "PrivacyDBPC")
Privacy:RegisterDefaults("profile", {
  VIP = {},                           -- list of all VIP
  AutoAnswer = {                      -- AutoAnswer text and [enabled] (not yet implemented)
    enabled = true,
    text = L["AUTO_ANSWER_TEXT"],
  },
  tmpVIP = 120,                       -- time for tmpVIP
  MultiWhisperDelay = 2.0,            -- delay for MultiWhispers
  allowGroup = true,                  -- allow group members to whisper
  allowRaid = true,                   -- ...   raid  ...
  allowGuild = true,                  -- ...
  allowFriends = true,                -- ...
  onlyInRange = false,                -- only allow whispers from players who are in 10 yards arround the player
                                      -- !currently tested!
  tellPrefix = "TELL",        
})

------

function Privacy:OnInitialize()  
  self:RegisterEvent("PARTY_MEMBERS_CHANGED", "RefreshGroup")
  self:RegisterEvent("RAID_ROSTER_UPDATE", "RefreshGroup")
  self:RegisterEvent("GUILD_ROSTER_UPDATE", "RefreshGuild")
  self:RegisterEvent("FRIENDLIST_UPDATE", "RefreshFriends")
  
  -- to disable an interface just comment it out in the toc, rename the *Init() or rename the file.
  if self.ConsoleInit then self:ConsoleInit() end
  if self.GuiInit then self:GuiInit() end
  if self.PluginInit then self:PluginInit() end
end

function Privacy:OnEnable() 
  if self:CheckSpamSentry() then
    self:Hook(SS, "ChatFrame_MessageEventHandler", "SpamSentry_MessageEventHandler")
  else
    self:Hook("ChatFrame_MessageEventHandler")
  end
  
  self:RefreshGroup()
  self:RefreshGuild()
  self:RefreshFriends()
  
  if PrivacyFu then
    PrivacyFu:UpdateText()
  end
end

function Privacy:OnDisable()
--  self:ListBlockedWhispers()
--  self:ClearBlocked()
  if PrivacyFu then
    PrivacyFu:UpdateText()
  end
end

------

--[[
    HOOKED ChatFrame_MessageEventHandler(event)
    
    Parameters:
      (string) event - the event
      
    Returns
      original return value
      
    Handles whispers and whisper-informs to block/pass.
--]]
function Privacy:ChatFrame_MessageEventHandler(event)
  if not self:DispatchChatEvent(event) then
    return --block/ignore
  end
  
  return self.hooks.ChatFrame_MessageEventHandler(event)
end

--[[
    HOOKED SpamSentry_MessageEventHandler(event, handler)
    
    Parameters:
      (string) event - the event
      (unknown) handler -  for SpamSentry
      
    Returns
      SpamSentry's return value
      
    Handles whispers and passes unblocked messages to SpamSentry.
--]]
function Privacy:SpamSentry_MessageEventHandler(SpamSentry, event, handler)
  if not self:DispatchChatEvent(event) then
    return --block/ignore
  end  
  
  return self.hooks[SS].ChatFrame_MessageEventHandler(SS, event, handler)
end

------

--[[
    :DispatchChatEvent(event)
    
    Parameters:
      (string) event - The event
    
    Returns:
      (boolean) block - Block the event or not
      
    Main part of the addon, blocks unwanted whispers. For use in hooked ChatFrame_OnEvent
--]]
function Privacy:DispatchChatEvent(event)
  if event == "CHAT_MSG_WHISPER" then
    local sender = arg2 or ""
    local msg = arg1    or ""

    -- filter multiple whispers from same player with the same text
    if (sender == self.LTSender) and (msg == self.LTMsg) then
      local t = GetTime()
      if (t - self.LTTime) <= self.db.profile.MultiWhisperDelay then
        self.LTTime = t
        return false  -- ignore
      end      
    else
      self.LTSender = sender or ""
      self.LTMsg    = msg    or ""
      self.LTTime   = GetTime()
    end

    local sPos, ePos = strfind(msg, self.db.profile.tellPrefix)
    if (not sPos) or (sPos ~= 1) then -- private key word
      if not self:IsAllowed(sender) then -- block this whisper
        self:SendAutoAnswer(sender)
        self:AddBlockedWhisper(sender, msg)
        return false
      end     
    end

  elseif event == "CHAT_MSG_WHISPER_INFORM" then
    local rec = arg2 or ""
    local msg = arg1 or ""

    -- filter multiple informs
    if (rec == self.LIRec) and (msg == self.LIMsg) then
      local t = GetTime()
      assert(self.db.profile.MultiWhisperDelay, "FFS!")
      if (t - self.LITime) <= self.db.profile.MultiWhisperDelay then
        self.LITime = t       
        return false -- ignore
      end
    else
      self.LIRec  = rec or ""
      self.LIMsg  = msg or ""
      self.LITime = GetTime()
    end

    -- it's the auto anser, hide
    local sPos, ePos = strfind(msg, AUTO_ANSWER_LEAD_IN)
    if sPos and (sPos == 2) then
      return false
    end

    self:AddTempVIP(rec)
  end
  
  return true
end

--[[
    :RefreshGroup()
    
    Parameters:
      nothing
      
    Returns:
      nothing
    
    Scans the player's party and raid to build a list of all member's names.
--]]
function Privacy:RefreshGroup()
  if GetNumPartyMembers() > 1 then
    self.group = new()
  else
    if self.group then
      del(self.group)
    end
  end
  if GetNumRaidMembers() > 1 then
    self.raid = new()
  else
    if self.raid then
      del(self.raid)
    end
  end
  
  -- scan the party
  for i=1, GetNumPartyMembers() do
    local name = UnitName("party"..i)
    self.group[name] = true
  end
  
  -- scan the raid
  for i=1, GetNumRaidMembers() do
    local name = UnitName("raid"..i)
    self.raid[name] = true
  end
end

--[[
    :RefreshGuild()
    
    Parameters:
      nothing
      
    Returns:
      nothing
    
    Scans the player's guild to build a list of all member's names.
--]]
function Privacy:RefreshGuild()  
  if self.guild then
    del(self.guild)
  end
  self.guild = new()
  
  for i=1, GetNumGuildMembers(false) do
    local name = GetGuildRosterInfo(i)
    self.guild[name] = true
  end
end

--[[
    :RefreshFriends()
    
    Parameters:
      nothing
      
    Returns:
      nothing
    
    Scans the player's guild to build a list of all member's names.
--]]
function Privacy:RefreshFriends()

  if self.friends then
    del(self.friends)
  end
  self.friends = new()
  
  for i=1, GetNumFriends() do
    local name = GetFriendInfo(i)
    
    if name then 
      self.friends[name] = true
    end
  end
end

--[[
    :IsAllowed(name)
    
    Parameters:
      (string) name - Unit's name
     
    Returns:
      (boolean) allowed - true: Unit is allowed, false: Unit is not allowed
      
    Checks whether a unit is allowed to send whispers. 
--]]
function Privacy:IsAllowed(name)
  if(TEXT(getglobal("CHAT_FLAG_"..tostring(arg6)) == CHAT_FLAG_GM)) then return true end  -- GMs are allways allowed!

  if self.db.profile.onlyInRange then 
    local r = self:GetRangeInfo(name)
    if r == 0 then  -- unit is out of range
      return false
    elseif r == 1 then  -- unit is in range
      return true
    end -- any other value: no unit id found
  end

  if self.db.profile.VIP[name] then
    return true
  end  
  
  if self.group and self.db.profile.allowGroup and self.group[name] then
    return true
  end
  
  if self.raid and self.db.profile.allowRaid and self.raid[name] then
    return true
  end
  
  if self.guild and self.db.profile.allowGuild and self.guild[name] then
    return true
  end
  
  if self.friends and self.db.profile.allowFriends and self.friends[name] then
    return true
  end
  
  local time = self.tmpVIP[name] 
  if not time then
    return false
  end
  
  if time >= GetTime() then
    return true
  end
  
  self.tmpVIP[name] = nil -- clearing
  return false
end

--[[ 
    :AddTempVIP(name)
    
    Parameters:
      (string) name - the temprorary VIP
    
    Returns:
      nothing
    
    Adds a new tmpVIP.
--]]
function Privacy:AddTempVIP(name)
  self.tmpVIP[name] = GetTime() + self.db.profile.tmpVIP
end

--[[
    :AddBlockedWhisper(name, text)
    
    Parameters:
      (string) name - Sender
      (string) text - Message
    
    Returns
      nothing
     
    Adds a whisper to the blocked whispers list.
--]]
function Privacy:AddBlockedWhisper(name, text)
  self:Print(L["Whisper blocked."])
  
  if not self.BlockedWhispers then
    self.BlockedWhispers = new()
  end
  tinsert(self.BlockedWhispers, (self:TimeStamp() .. " [" .. name .. "]: ".. text))
  
  if PrivacyFu then
    PrivacyFu:UpdateText()
  end
end

--[[
    :TimeStamp()
    
    Parameters:
      nothing
      
    Returns:
      (string) TimeStamp
      
    Returns a formated timestamp.
--]]
function Privacy:TimeStamp()
  local h, min = GetGameTime()
  
  h = (h < 10) and ("0"..h) or h
  min = (min < 10) and ("0"..min) or min
  
  return ("["..h .. ":" .. min.."]")
end

--[[
    :CheckSpamSentry()
    
    Parameters:
      nothing
      
    Returns:
      (boolean) SpamSentry loaded
      
    Checks whether the addon SpamSentry is loaded.
--]]
local SSLoaded = nil
function Privacy:CheckSpamSentry()
  if not SSLoaded then
    SSLoaded = IsAddOnLoaded("SpamSentry")
  end
  return SSLoaded
end

--[[
    :ConvertOldDB() UNTESTED
    
    Parameters:
      nothing
      
    Returns:
      nothing
      
    Converts old Privacy saved variables to the new AceDB.
--]]
function Privacy:ConvertOldDB()
  local realm  = GetRealmName()
  local player = UnitName("player")

  if PrivacyFilterState then
    if PrivacyFilterState[realm] then
      if PrivacyFilterState[realm][player] ~= nil then
        if PrivacyFilterState[realm][player] == 0 then
          self.db.profile.enabled = false
        else
          self.db.profile.enabled = true
        end
        PrivacyFilterState[realm][player] = nil
      end
    end
  end
  
  if PrivacyVIPs then
    if PrivacyVIPs[realm] then
      if PrivacyVIPs[realm][player] then
        for i=1, #(PrivacyVIPs[realm][player]) do
          self.db.profile.VIP[PrivacyVIPs[realm][player][i]] = true
        end
        PrivacyVIPs[realm][player] = nil
      end
    end
  end
  
  if PrivacyAutoAnswer then
    if PrivacyAutoAnswer[realm] then
      if PrivacyAutoAnswer[realm][player] then
        self.db.profile.AutoAnswer.text = PrivacyAutoAnswer[realm][player]
        PrivacyAutoAnswer[realm][player] = nil
      end
    end
  end
end

--[[
    :SendAutoAnswer(name)
    
    Parameters:
      (string) name - Target
    
    Returns:
      nothing
      
    Sends the AutoAnswer to the player namend name.
--]]
function Privacy:SendAutoAnswer(name)
  if self.db.profile.AutoAnswer.enabled then
    --if UnitName("player") == sender then return end
    SendChatMessage(AUTO_ANSWER_LEAD_IN .. self.db.profile.AutoAnswer.text, "WHISPER", nil, name)
  end  
end

------

local UnitName = UnitName

--[[
    :UnitIdByName(name)
    
    Parameters:
      (string) name - The unit's name
     
    Returns
      (unitId) id - The unitId for the given name or nil, if no unitId is found
      
    Trys to get a unitId for a name.
--]]
function Privacy:UnitIdByName(name)
  if name == UnitName("target") then
    return "target"
  elseif name == UnitName("targettarget") then
    return "targettarget"
  elseif name == UnitName("focus") then
    return "focus"
  elseif name == UnitName("focustarget") then
    return "focustarget"
  elseif name == UnitName("mouseover") then
    return "mouseover"
  elseif name == UnitName("mouseovertarget") then
    return "mouseovertarget"
  else
    for i=1, GetNumPartyMembers() do
      local u = "party"..i
      if name == UnitName(u) then
        return (u)
      end
      u = u .. "target"
      if name == UnitName(u) then
        return (u)
      end
    end
    for i=1, GetNumRaidMembers() do
      local u = "raid"..i
      if name == UnitName(u) then
        return (u)
      end
      u = u .. "target"
      if name == UnitName(u) then
        return (u)
      end
    end
  end
  return nil
end

--[[
    :GetRangeInfo(name)
    
    Parameters:
      (string) name - The unit's name
      
    Returns
      (int) rangeInfo - 0: out of range, 1 - in 10 yards, 2 - no unit id
      
    Uses :UnitIdByName and CheckInteractDistance to get the units range.
    Returns true if the name is not found, or the unit is in "duel" range (10 yards, lowest possible range)
--]]
function Privacy:GetRangeInfo(name)
  local id = self:UnitIdByName(name)
  
  if not id then
    return 2
  end
  
  if CheckInteractDistance(id, 3) then -- 3 means "duell", 10 yards
    return 1
  else
    return 0
  end
end

------

-- following functions are called by the different interfaces (Console/GUI/Plugin)

function Privacy:PrintStatus()
  if self:IsActive() then
    self:Print(L["Enabled"])
  else
    self:Print(L["Disabled"])
  end
end

function Privacy:ListVIP()
  self:Print(L["VIP list:"])
  for name, flag in pairs(self.db.profile.VIP) do
    if flag then
      self:Print(name)
    end
  end
  self:Print(L["Temporary VIP list:"])
  for name, flag in pairs(self.tmpVIP) do
    if flag then
      self:Print(" ", name)
    end
  end
end

function Privacy:ListBlockedWhispers()
  self:Print(L["Blocked whispers:"])
  for _, text in pairs(self.BlockedWhispers) do
    self:Print(text)
  end
end

function Privacy:SetAutoAnswer(text)
  if (text == "DEFAULT") or (text == "") then
    text = L["AUTO_ANSWER_TEXT"]
  end
  text = gsub(text, REPLACE_BY_TELL_PREF, self.db.profile.tellPrefix)
  self.db.profile.AutoAnswer.text = text
  self:Print(format(L["AutoAnswer text is now %s."], text))
end

function Privacy:AddVIP(name)
  self.db.profile.VIP[name] = true
  self:Print(format(L["Added %s to the VIP list."], name))
end

function Privacy:RemoveVIP(name)
  self.RemoveName = name
  StaticPopup_Show("PRIVACY2_CONFIRM_VIP_REMOVE", name, "", name)  
end

function Privacy:OpenConfig()
  Waterfall:Open(self.name) 
end

function Privacy:GetNumBlockedWhispers()
  return #(self.BlockedWhispers)
end

function Privacy:ClearBlocked()
  if self.BlockedWhispers then
    del(self.BlockedWhispers)
  end
  self.BlockedWhispers = new()
end

function Privacy:SetTellPrefix(pre)
  if not pre or pre == "" then
    pre = "TELL"
  end
  self.db.profile.tellPrefix = pre
  self:Print(format(L["Your private keyword is now %s."], pre))
end

function Privacy:GetTellPrefix()
  return self.db.profile.tellPrefix
end

------

-- Popup to confirm VIPRemove

StaticPopupDialogs["PRIVACY2_CONFIRM_VIP_REMOVE"] = {
  text = L["Remove %s from VIP list?"],
  button1 = YES,
  button2 = NO,
  OnAccept = function()
    Privacy:ConfirmedRemove()
  end,
  timeout = 0,
  showAlert = 1,
  hideOnEscape = 1,
}
Privacy.Confirm = StaticPopupDialogs["PRIVACY2_CONFIRM_VIP_REMOVE"]

function Privacy:ConfirmedRemove()
  if self.RemoveName then
    self.db.profile.VIP[self.RemoveName] = nil
    self:Print(format(L["Removed %s from VIP list."], self.RemoveName))
    self.RemoveName = nil
  end
end
  