--[[
    GUI.lua
    
    GUI configuration for Privacy2 using Waterfall-1.0.
    
    The original Privacy is by Sterntraeumer.
--]]

assert(Privacy, "Privacy2 must be loaded to create the gui. Please report this bug.")

local L = AceLibrary("AceLocale-2.2"):new("Privacy2")

local Waterfall = AceLibrary("Waterfall-1.0")

------

--Privacy.CurrentVIP = 1

Privacy.Gui = {
  name = "Privacy2",
  type = "group",
  icon = "Interface\\Icons\\Ability_Warrior_Revenge",
  handler = Privacy,
  args = {
    VIP = {   -- VIP Management
      type = "group",
      name = L["VIP"],
      order = 1,
      --icon = "Interface\\Icons\\Misc_GroupLooking",   -- icons are not used by Waterfall, but maybe in the future
      args = {
        add = {
          name = L["Add"],
          desc = L["Adds a VIP."],
          type = "text",
          get  = false,
          set  = "AddVIP",
          order = 1,
        },
        remove = {
          name = L["Remove"],
          desc = L["Removes a VIP."],
          type = "text",
          get  = false,
          set  = "RemoveVIP",
          order = 2,
        },
        sp1= {
          type = "header",
          name = " ",
          order = 3,
        },
        list = {
          name = L["Show VIP"],
          desc = L["Lists all VIP."],
          type = "execute",
          func = "ListVIP",
          order = 4,
        },
      },
    },   
    Misc = {  -- Misc Management
      type = "group",
      name = L["Misc"],
      order = 2,
      --icon = "Interface\\Icons\\Misc_Bomb_04",
      args = {
        group = {
          type = "toggle",
          name = L["Group whispers"],
          desc = L["Allow group members to whisper."],
          get  = function() return Privacy.db.profile.allowGroup end,
          set  = function(v) Privacy.db.profile.allowGroup = v end,
          order = 1,
        },
        raid = {
          type = "toggle",
          name = L["Raid whispers"],
          desc = L["Allow raid members to whisper."],
          get  = function() return Privacy.db.profile.allowRaid end,
          set  = function(v) Privacy.db.profile.allowRaid = v end,
          order = 2,
        },
        guild = {
          type = "toggle",
          name = L["Guild whispers"],
          desc = L["Allow guild members to whisper."],
          get  = function() return Privacy.db.profile.allowGuild end,
          set  = function(v) Privacy.db.profile.allowGuild = v end,
          order = 3,
        },
        friends = {
          type = "toggle",
          name = L["Friend whispers"],
          desc = L["Allow friends to whisper."],
          get  = function() return Privacy.db.profile.allowFriends end,
          set  = function(v) Privacy.db.profile.allowFriends = v end,
          order = 4,
        },
        clear = {
          type = "execute",
          name = L["Clear"],
          desc = L["Clears all blocked whispers."],
          func = "ClearBlocked",
          order = 5,
        },
      },
    },
    -- Main settings
    active = {
      type = "toggle",
      name = L["Enabled"],
      desc = L["Enables/disables the addon."],
      get  = "IsActive",
      set  = "ToggleActive",
      order = 1,
    },
    sp1= {
      type = "header",
      name = " ",
      order = 2,
    },
    setanswer = {
      name  = L["Auto Answer"],
      desc  = L["Sets the auto answer for blocked whispers. '$k' will be replaced with your private keyword.\n\nYour current auto answer is:\n%s"],
      type  = "text",
      usage = L["<text>"],
      get   = "GuiGetAutoAnswer",
      set   = "SetAutoAnswerText", 
      order = 3,
    },
    sp2 = {
      type = "header",
      name = " ",
      order = 4,
    },
    replay = {
      name = L["Replay"],
      desc = L["Lists all blocked whispers."],
      type = "execute",
      func = "ListBlockedWhispers",
      order = 5,
    },
    sp3 = {
      type = "header",
      name = " ",
      order = 6,
    },
    keyword = {
      name = L["Keyword"],
      desc = L["Change your private keyword."],
      type = "text",
      get  = "GetTellPrefix",
      set  = "SetTellPrefix",
      order = 7,
    },
  },
}

------

-- Main

function Privacy:GuiGetAutoAnswer()
  self.Gui.args.setanswer.desc = format(L["Sets the auto answer for blocked whispers. '$k' will be replaced with your private keyword.\n\nYour current auto answer is:\n%s"],
                                        self.db.profile.AutoAnswer.text)

  return self.db.profile.AutoAnswer.text or "Error! Cannot find the AutoAnswer text. Please report this bug."
end

------

function Privacy:GuiInit()
  Waterfall:Register(self.name,
    "aceOptions", self.Gui,
    "title", format(L["Privacy2 Config (%s)"], self.Revision),
    "colorR", 0.1,
    "colorG", 0.5,
    "colorB", 0.9
    )
end