--[[
    Plugin.lua
    
    FuBar plugin for Privacy2.
    
    The original Privacy is by Sterntraeumer.
--]]

assert(Privacy, "Privacy2 must be loaded to create the FuBar plugin. Please report this bug.")

local L = AceLibrary("AceLocale-2.2"):new("Privacy2")
local Tablet = AceLibrary("Tablet-2.0")

PrivacyFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0")

------

PrivacyFu.Options = {
  type = "group",
  handler = PrivacyFu,
  args = {
    config = {
      name = L["Configuration"],
      desc = L["Shows the config interface."],
      type = "execute",
      func = function() Privacy:OpenConfig() end,
    },
  },
}

------

PrivacyFu.hasIcon = "Interface\\Icons\\Ability_Warrior_Revenge"
PrivacyFu.hasNoText = false
PrivacyFu.defaultPosition = "LEFT"
PrivacyFu.clickableTooltip = false
PrivacyFu.OnMenuRequest = PrivacyFu.Options
PrivacyFu.defaultMinimapPosition = 180

------

function PrivacyFu:OnClick()
  Privacy:ListBlockedWhispers()
end

function PrivacyFu:OnDoubleClick()
  Privacy:ToggleActive()
end

function PrivacyFu:OnDataUpdate()
  self.NumBlocked = Privacy:GetNumBlockedWhispers()
  self.PActive    = Privacy:IsActive()
end

function PrivacyFu:OnTooltipUpdate()
  self:UpdateData()

  local cat
  
  cat = Tablet:AddCategory(
    'columns', 2
  )
  
  cat:AddLine(
    "text", L["Status:"],
    "text2", L["ACTIVE_MAP"][self.PActive]
  )
  
  cat:AddLine(
    "text", L["Blocked Whispers:"],
    "text2", tostring(self.NumBlocked) or "0"
  )
  
  if self.PActive then
    Tablet:SetHint(L["HINT_ENABLED"])
  else
    Tablet:SetHint(L["HINT_DISABLED"])
  end
end

function PrivacyFu:OnTextUpdate()
  self:UpdateData()
  
  local text = L["Blocked: "] .. self.NumBlocked or "0"
  
  self:SetText(text)
end

------

function PrivacyFu:OnInitialize()

end

function PrivacyFu:OnEnable()

end

function PrivacyFu:OnDisable()

end

------

function Privacy:PluginInit()
  self.Plugin = PrivacyFu
  PrivacyFu.db = self:AcquireDBNamespace("FuBarPlugin")
end

